#
# Copyright (c) 2018-2025 Charles Godwin <magnum@godwin.ca>
#
# SPDX-License-Identifier:    BSD-3-Clause
#
#  BUILDINFO
__version__="2.0.7"
#
# names of devices
#
ACLD = "ACLD"
AGS = "AGS"
BMK = "BMK"
INVERTER = "INVERTER"
PT100 = "PT100"
REMOTE = "REMOTE"
RTR = "RTR"
UNKNOWN = "UNKNOWN"
#
#  Names of all the packet types
#
AGS_A1 = "AGS_A1"
AGS_A2 = "AGS_A2"

BMK_81 = "BMK_81"

INV = "INVERTER"
INV_C = "INVERTER_C"

PT_C1 = "PT_C1"
PT_C2 = "PT_C2"
PT_C3 = "PT_C3"

REMOTE_C = "REMOTE_C"
REMOTE_00 = "REMOTE_00"
REMOTE_11 = "REMOTE_11"
REMOTE_80 = "REMOTE_80"
REMOTE_A0 = "REMOTE_A0"
REMOTE_A1 = "REMOTE_A1"
REMOTE_A2 = "REMOTE_A2"
REMOTE_A3 = "REMOTE_A3"
REMOTE_A4 = "REMOTE_A4"
REMOTE_C0 = "REMOTE_C0"
REMOTE_C1 = "REMOTE_C1"
REMOTE_C2 = "REMOTE_C2"
REMOTE_C3 = "REMOTE_C3"
REMOTE_D0 = "REMOTE_D0"

RTR_91 = "RTR_91"

ACLD_D1 = "ACLD_D1"
